export declare abstract class BlockTipBase<B> {
    data: B;
    constructor(data: B);
    /**
     * Block number sometimes refers to as block height or ledger height (number of block in blockchain)
     */
    abstract get number(): number;
    /**
     * Block hash directly from underlying node
     */
    abstract get blockHash(): string;
    /**
     * Flare standardized block hash (hex encoded string of length 64 (32 bytes) without 0x prefix)
     */
    abstract get stdBlockHash(): string;
    /**
     * Only return something on Tips
     */
    get chainTipStatus(): string;
}
export declare abstract class BlockHeaderBase<B> extends BlockTipBase<B> {
    /**
     * Previous block hash directly from underlying node
     */
    abstract get previousBlockHash(): string;
    /**
     * Flare standardized previous block hash (hex encoded string of length 64 (32 bytes) without 0x prefix)
     */
    abstract get stdPreviousBlockHash(): string;
    /**
     * block timestamp as unix time (elapsed seconds since 1.1.1970)
     */
    abstract get unixTimestamp(): number;
    /**
     * Number of transactions in block
     */
    abstract get transactionCount(): number;
}
export declare abstract class BlockBase<B> extends BlockHeaderBase<B> {
    /**
     * Array of transaction ids of all transactions in block
     */
    abstract get transactionIds(): string[];
    /**
     * Array of flare standardized transaction ids in block (hex encoded string of length 64 (32 bytes) without 0x prefix)
     */
    abstract get stdTransactionIds(): string[];
    /**
     * Return if block is valid
     * Mainly for XRP
     */
    get isValid(): boolean;
}
/**
 * Base class for blocks that also include all transactions (including information about the transaction)
 */
export declare abstract class FullBlockBase<B, T> extends BlockBase<B> {
    /**
     * Array of transactions objects in block
     */
    abstract get transactions(): T[];
}
export declare type IBlockTip = BlockTipBase<any>;
export declare type IBlockHeader = BlockHeaderBase<any>;
export declare type IBlock = BlockBase<any>;
export declare type IFullBlock = FullBlockBase<any, any>;
/**
 * Algo and Ripple (XRP) have no specific block header endpoint (liter block with limited data)
 */
export { BtcBlockHeader } from "./blockHeaders/BtcBlockHeader";
export { DogeBlockHeader } from "./blockHeaders/DogeBlockHeader";
export { LtcBlockHeader } from "./blockHeaders/LtcBlockHeader";
export { UtxoBlockHeader } from "./blockHeaders/UtxoBlockHeader";
export { AlgoBlock } from "./blocks/AlgoBlock";
export { BtcBlock } from "./blocks/BtcBlock";
export { DogeBlock } from "./blocks/DogeBlock";
export { LtcBlock } from "./blocks/LtcBlock";
export { UtxoBlock } from "./blocks/UtxoBlock";
export { XrpBlock } from "./blocks/XrpBlock";
export { BtcBlockTip } from "./blockTips/BtcBlockTip";
export { DogeBlockTip } from "./blockTips/DogeBlockTip";
export { LtcBlockTip } from "./blockTips/LtcBlockTip";
export { UtxoBlockTip } from "./blockTips/UtxoBlockTip";
export { BtcFullBlock } from "./fullBlocks/BtcFullBlock";
export { DogeFullBlock } from "./fullBlocks/DogeFullBlock";
export { LtcFullBlock } from "./fullBlocks/LtcFullBlock";
export { UtxoFullBlock } from "./fullBlocks/UtxoFullBlock";
export { XrpFullBlock } from "./fullBlocks/XrpFullBlock";
//# sourceMappingURL=BlockBase.d.ts.map