/// <reference types="node" />
import { EncodedTransaction } from "algosdk";
import { Asset } from "algosdk/dist/types/client/v2/algod/models/types";
import { AlgoTransaction } from "../base-objects/TransactionBase";
import { RateLimitOptions } from "../types";
import { IIGetBlockRes, IIGetTransactionRes, MccLoggingOptions } from "./genericMccTypes";
export declare class AlgoMccCreate {
    algod: AlgoNodeApp;
    apiTokenKey?: string;
    indexer?: AlgoNodeApp | undefined;
    inRegTest?: boolean | undefined;
    rateLimitOptions?: RateLimitOptions | undefined;
    loggingOptions?: MccLoggingOptions | undefined;
}
export declare class AlgoNodeApp {
    url: string;
    token: string;
}
export interface IAlgoStatusRes {
    catchpoint?: string;
    catchpointAcquiredBlocks?: number;
    catchpointProcessedAccounts?: number;
    catchpointTotalAccounts?: number;
    catchpointTotalBlocks?: number;
    catchpointVerifiedAccounts?: number;
    catchupTime: number;
    lastCatchpoint?: string;
    lastRound: number;
    lastVersion: string;
    nextVersion: string;
    nextVersionRound: number;
    nextVersionSupported: boolean;
    stoppedAtUnsupportedRound: boolean;
    timeSinceLastRound: number;
}
export interface IAlgoBlockHeaderData {
    earn: number;
    fees: string;
    frac: number;
    gen: string;
    gh: string;
    prev: string;
    proto: string;
    rnd: number;
    rwcalr: number;
    rwd: string;
    seed: string;
    tc: number;
    ts: number;
    txn: string;
    txns: any[];
}
export interface IAlgoBlockData {
    genesisHash: string;
    genesisId: string;
    previousBlockHash: string;
    rewards?: any;
    round: number;
    seed: string;
    timestamp: number;
    transactions?: IAlgoTransaction[];
    transactionsRoot: string;
    txnCounter?: number;
    upgradeState: any;
    upgradeVote: any;
}
interface IAlgoCertProp {
    dig: Buffer;
    encdig: Buffer;
    oprop: Buffer;
}
interface IAlgoIndexerCertProp {
    dig: string;
    encdig: string;
    oprop: string;
}
export interface IAlgoCert {
    prop: IAlgoCertProp;
    rnd: number;
    step: number;
    vote: any[];
}
export interface IAlgoIndexerCert {
    prop: IAlgoIndexerCertProp;
    rnd: number;
    step: number;
    vote: any[];
}
export interface IAlgoIndexerAsset {
    createdAtRound?: number;
    deleted?: boolean;
    destroyedAtRound?: number;
    index: number;
    params: {
        clawback?: string;
        creator: string;
        decimals: number;
        defaultFrozen?: boolean;
        freeze?: string;
        manager?: string;
        metadataHash?: string;
        name?: string;
        nameB64?: string;
        reserve?: string;
        total: number;
        unitName?: string;
        unitNameB64?: string;
        url?: string;
        urlB64?: string;
    };
}
export interface IAlgoGetBlockRes extends IAlgoBlockData, IIGetBlockRes {
    cert: IAlgoCert;
    type: "IAlgoGetBlockRes";
}
export interface IAlgoGetIndexerBlockRes extends IAlgoBlockData, IIGetBlockRes {
    cert: IAlgoIndexerCert;
}
export interface IAlgoGetBlockHeaderRes {
    type: "IAlgoGetBlockHeaderRes";
    block: IAlgoBlockHeaderData;
    cert: IAlgoCert;
}
export interface IAlgoBlockMsgPackBlock {
    earn: number;
    fees: Buffer;
    frac: number;
    gen: string;
    gh: Buffer;
    prev: Buffer;
    proto: string;
    rnd: number;
    rwcalr: number;
    rwd: Buffer;
    seed: Buffer;
    tc: number;
    ts: number;
    txn: Buffer;
    txns?: any[];
}
export interface IAlgoBlockMsgPack {
    block: IAlgoBlockMsgPackBlock;
    cert: IAlgoCert;
}
export interface IAlgoBlockAdditionalData {
    transactionsObjects: AlgoTransaction[];
}
export interface IAlgoSignature {
    sig: string;
}
export interface IAlgoPaymentTransaction {
    amount: number;
    closeAmount?: number;
    closeRemainderTo?: string;
    receiver: string;
}
/**
 * Docs from : https://developer.algorand.org/docs/rest-apis/indexer/#transaction
 */
export interface IAlgoTransaction {
    applicationTransaction?: any;
    assetConfigTransaction?: any;
    assetFreezeTransaction?: any;
    assetTransferTransaction?: any;
    authAddr?: string;
    closeRewards?: number;
    closingAmount?: number;
    confirmedRound?: number;
    createdApplicationIndex?: number;
    fee: number;
    firstValid: number;
    genesisHash?: string;
    genesisId?: string;
    globalStateDelta?: any;
    group?: string;
    id?: string;
    innerTxns?: IAlgoTransaction[];
    intraRoundOffset?: number;
    keyregTransaction?: any;
    lastValid: number;
    lease?: string;
    localStateDelta?: any;
    logs?: string[];
    note?: string;
    paymentTransaction?: IAlgoPaymentTransaction;
    receiverRewards?: number;
    rekeyTo?: string;
    roundTime?: number;
    sender: string;
    senderRewards?: number;
    stateProofTransaction?: any;
    signature?: IAlgoSignature;
    txType: AlgoTransactionTypeOptions;
}
export declare type IAlgoTransactionMsgPack = EncodedTransaction & {
    txid: string;
    timestamp: number;
    hgi: boolean;
    sig?: Buffer;
    lsig?: Buffer;
    msig?: Buffer;
    sgnr?: Buffer;
};
export declare type AlgoTransactionTypeOptions = "pay" | "keyreg" | "acfg" | "axfer" | "afrz" | "appl" | "pay_close" | "axfer_close";
export interface IAlgoLitsTransaction {
    address?: string;
    addressRole?: "sender" | "receiver" | "freeze-target";
    afterTime?: string;
    beforeTime?: string;
    currencyGreaterThan?: number;
    currencyLessThan?: number;
    excludeCloseTo?: boolean;
    applicationId?: number;
    assetId?: number;
    limit?: number;
    maxRound?: number;
    minRound?: number;
    round?: number;
    next?: string;
    notePrefix?: string;
    rekeyTo?: boolean;
    sigType?: "sig" | "msig" | "lsig";
    txType?: "pay" | "keyreg" | "acfg" | "axfer" | "afrz" | "appl";
    txid?: string;
}
export interface IAlgoGetTransactionRes extends IIGetTransactionRes {
    currentRound: number;
    transaction: IAlgoTransaction;
}
export interface IAlgoGetFullTransactionRes extends IAlgoTransaction, IIGetTransactionRes {
    currentRound: number;
}
export interface IAlgoListTransactionRes {
    currentRound: number;
    nextToken?: string;
    transactions: IAlgoTransaction[];
}
export interface IAlgoGetStatus {
    catchpoint?: string;
    catchpointAcquiredBlocks?: number;
    catchpointProcessedAccounts?: number;
    catchpointTotalAccounts?: number;
    catchpointTotalBlocks?: number;
    catchpointVerifiedAccounts?: number;
    catchupTime: number;
    lastCatchpoint?: string;
    lastRound: number;
    lastVersion: string;
    nextVersion: string;
    nextVersionRound: number;
    nextVersionSupported: boolean;
    stoppedAtUnsupportedRound: boolean;
    timeSinceLastRound: number;
}
export interface IAlgoBuildVersion {
    branch: string;
    buildNumber: number;
    channel: string;
    commitHash: string;
    major: number;
    minor: number;
}
export interface IAlgoVersion {
    build: IAlgoBuildVersion;
    genesisHashB64: string;
    genesisId: string;
    versions: string[];
}
export interface IAlgoStatusObject {
    health: number;
    status: IAlgoGetStatus;
    versions: IAlgoVersion;
}
export interface IAlgoHexAddress {
    publicKey: string;
    checksum: string;
}
export declare type IAlgoAssets = Asset;
export interface IAlgoAdditionalData {
    assetInfo?: IAlgoAssets;
}
export interface IAlgoIndexerAdditionalData {
    assetInfo?: IAlgoIndexerAsset;
}
export {};
//# sourceMappingURL=algoTypes.d.ts.map