"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.utxo_ensure_data = exports.utxo_check_expect_block_out_of_range = exports.utxo_check_expect_empty = exports.recursive_block_tip = exports.recursive_block_hash = exports.WordToOpcode = void 0;
const UtxoBlockTip_1 = require("../base-objects/blockTips/UtxoBlockTip");
const utils_1 = require("./utils");
/**
 * Opcode words to their opcodes
 * @dev https://en.bitcoin.it/wiki/Script
 */
var WordToOpcode;
(function (WordToOpcode) {
    WordToOpcode[WordToOpcode["OP_RETURN"] = 106] = "OP_RETURN";
})(WordToOpcode = exports.WordToOpcode || (exports.WordToOpcode = {}));
async function recursive_block_hash(clinet, hash, processHeight) {
    if (hash === "") {
        return [];
    }
    if (processHeight <= 1) {
        return [hash];
    }
    else {
        const Cblock = await clinet.getBlockHeader(hash);
        const hs = Cblock.previousBlockHash;
        return (await recursive_block_hash(clinet, hs, processHeight - 1)).concat([hash]);
    }
}
exports.recursive_block_hash = recursive_block_hash;
async function recursive_block_tip(clinet, tip, processHeight) {
    if (tip.stdBlockHash === "") {
        return [];
    }
    const tempTip = new UtxoBlockTip_1.UtxoBlockTip({ hash: tip.stdBlockHash, height: tip.number, branchlen: tip.data.branchlen, status: tip.data.status });
    if (processHeight <= 1) {
        return [tempTip];
    }
    else {
        const CurrBlock = await clinet.getBlockHeader(tip.stdBlockHash);
        const previousHash = CurrBlock.previousBlockHash;
        const previousHeight = CurrBlock.number - 1;
        return (await recursive_block_tip(clinet, new UtxoBlockTip_1.UtxoBlockTip({ hash: previousHash, height: previousHeight, branchlen: tip.data.branchlen, status: tip.chainTipStatus }), processHeight - 1)).concat([tempTip]);
    }
}
exports.recursive_block_tip = recursive_block_tip;
////////////////////////////
//// MCC Error handling ////
////////////////////////////
/**
 * Check if return error code is -5 (can't find transaction)
 * @param data
 * @returns
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function utxo_check_expect_empty(data) {
    if (!data || !data.error || !data.error.code) {
        return false;
    }
    if (data.error.code && data.error.code === -5) {
        return true;
    }
    else {
        return false;
    }
}
exports.utxo_check_expect_empty = utxo_check_expect_empty;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function utxo_check_expect_block_out_of_range(data) {
    if (data && data.error && data.error.code) {
        if (data.error.code === -8) {
            return true;
        }
        if (data.error.code === -1) {
            return true;
        }
        else {
            return false;
        }
    }
    else {
        return false;
    }
}
exports.utxo_check_expect_block_out_of_range = utxo_check_expect_block_out_of_range;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function utxo_ensure_data(data) {
    if (data === null || data === void 0 ? void 0 : data.error) {
        throw (0, utils_1.MccError)(data);
    }
}
exports.utxo_ensure_data = utxo_ensure_data;
//# sourceMappingURL=utxoUtils.js.map