"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const index_1 = require("../../src/index");
const testUtils_1 = require("../testUtils");
const sinon_1 = __importDefault(require("sinon"));
const XRPMccConnection = {
    url: process.env.XRP_URL || "",
    username: process.env.XRP_USERNAME || "",
    password: process.env.XRP_PASSWORD || "",
    apiTokenKey: process.env.FLARE_API_PORTAL_KEY || "",
};
describe(`XRP transactions in full block vs transactions from getTransaction (${(0, testUtils_1.getTestFile)(__filename)})`, () => {
    const blockNumbersToCheck = [75728627];
    before(function () {
        sinon_1.default.stub(console, "log");
    });
    after(function () {
        sinon_1.default.restore();
    });
    for (const blockNumber of blockNumbersToCheck) {
        describe(`Testing transactions in block ${blockNumber} (transitions from full block vs )`, () => {
            let client;
            let block;
            let fullBlock;
            before(async () => {
                client = new index_1.MCC.XRP(XRPMccConnection);
                block = await client.getBlock(blockNumber);
                fullBlock = await client.getFullBlock(blockNumber);
            });
            it("Block and Full block transaction count should be equal", () => {
                (0, chai_1.expect)(block.transactionCount).to.eq(fullBlock.transactionCount);
            });
            it("Block and Full block transaction ids should be equal", () => {
                (0, chai_1.expect)(block.transactionIds.sort()).to.deep.eq(fullBlock.transactionIds.sort());
            });
            it("Iterating over transactions", async () => {
                for (const transaction of fullBlock.transactions) {
                    console.log(transaction.txid);
                    const transObject = await client.getTransaction(transaction.txid);
                    (0, chai_1.expect)(transaction.txid).to.eq(transObject.txid);
                    (0, chai_1.expect)(transaction.stdTxid).to.eq(transObject.stdTxid);
                    (0, chai_1.expect)(transaction.hash).to.eq(transObject.hash);
                    (0, chai_1.expect)(transaction.reference.sort()).to.deep.eq(transObject.reference.sort());
                    (0, chai_1.expect)(transaction.stdPaymentReference).to.eq(transObject.stdPaymentReference);
                    (0, chai_1.expect)(transaction.unixTimestamp).to.eq(transObject.unixTimestamp);
                    (0, chai_1.expect)(transaction.sourceAddresses.sort()).to.deep.eq(transObject.sourceAddresses.sort());
                    (0, chai_1.expect)(transaction.receivingAddresses.sort()).to.deep.eq(transObject.receivingAddresses.sort());
                    // TODO: Uncomment when asset transactions are supported
                    //  expect(transaction.assetSourceAddresses.sort()).to.deep.eq(transObject.sourceAddresses.sort());
                    //  expect(transaction.assetReceivingAddresses.sort()).to.deep.eq(transObject.receivingAddresses.sort());
                    (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(transObject.fee.toNumber());
                    (0, chai_1.expect)((0, testUtils_1.AddressAmountEqual)(transaction.spentAmounts, transObject.spentAmounts)).to.eq(true);
                    (0, chai_1.expect)((0, testUtils_1.AddressAmountEqual)(transaction.receivedAmounts, transObject.receivedAmounts)).to.eq(true);
                    // TODO: Uncomment when asset transactions are supported
                    //  expect(AddressAmountEqual(transaction.assetSpentAmounts, transObject.assetSpentAmounts)).to.eq(true);
                    //  expect(AddressAmountEqual(transaction.assetReceivedAmounts, transObject.assetReceivedAmounts)).to.eq(true);
                    (0, chai_1.expect)(transaction.type).to.eq(transObject.type);
                    (0, chai_1.expect)(transaction.isNativePayment).to.eq(transObject.isNativePayment);
                    (0, chai_1.expect)(transaction.currencyName).to.eq(transObject.currencyName);
                    (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(transObject.elementaryUnits.toNumber());
                }
            }).timeout(1000 * 60 * 5);
        });
    }
});
//# sourceMappingURL=transactionsVsTransactionsInBlock.test.js.map